﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HarmonyLib;
using Enums;
using UnityEngine;
using System.Reflection;

namespace Bidon_Carry_All {
    /// <summary>
    /// Encapsulates the functionality to override soem of the liquid container implementation in the game.
    /// </summary>
    [HarmonyPatch(typeof(LiquidContainer), "UpdateSlotsActivity")]
    internal class Update_FillBidon_Patch {

        /// <summary>
        /// Prefix method when the liquid container class processor is used for pouring/filling.
        /// </summary>
        /// <param name="__instance">The original liquid container instance.</param>
        /// <returns>True if pour/fill is successful, otherwise false.</returns>
        [HarmonyPrefix]
        public static bool Prefix(LiquidContainer __instance) {
            Debug.Log("Update Slots Activity Override");

            var inInventory = __instance.m_InInventory;

            if (inInventory) {
                __instance.m_GetSlot.gameObject.SetActive(false);
                __instance.m_PourSlot.gameObject.SetActive(false);
                return false;
            }

            var getSlot = __instance.m_GetSlot;
            var pourSlot = __instance.m_PourSlot;
            var isCoconut = __instance.m_Info.m_ID == ItemID.Coconut;

            if (isCoconut) {
                __instance.m_GetSlot.gameObject.SetActive(false);
                __instance.m_PourSlot.gameObject.SetActive(false);
            }

            if (null != getSlot) {
                if (__instance.m_Info is LiquidContainerInfo lcInfo) {
                    if (lcInfo.m_Amount < 1f) {
                        getSlot.gameObject.SetActive(false);
                    }
                    else {
                        if (getSlot.gameObject.activeSelf) {
                            if (lcInfo.m_Amount < 1f) {
                                getSlot.gameObject.SetActive(false);
                            }
                            else {                                
                                if (getSlot.CanInsertItem(Inventory3DManager.Get().m_CarriedItem)) {
                                    LiquidContainerInfo liquidContainerInfo2 = (LiquidContainerInfo)Inventory3DManager.Get().m_CarriedItem.m_Info;

                                    if (liquidContainerInfo2.m_Amount >= liquidContainerInfo2.m_Capacity) {
                                        getSlot.gameObject.SetActive(false);
                                    }
                                }
                                else {
                                    getSlot.gameObject.SetActive(false);
                                }
                            }
                        }
                        else {                            
                            if (getSlot.CanInsertItem(Inventory3DManager.Get().m_CarriedItem)) {
                                
                                if (Inventory3DManager.Get().m_CarriedItem.m_Info.m_ID == ItemID.Coconut) {
                                    getSlot.gameObject.SetActive(false);
                                }
                                else {
                                    LiquidContainerInfo liquidContainerInfo3 = (LiquidContainerInfo)Inventory3DManager.Get().m_CarriedItem.m_Info;
                                    
                                    if (liquidContainerInfo3.m_Amount < liquidContainerInfo3.m_Capacity) {
                                        getSlot.gameObject.SetActive(true);
                                    }
                                }
                            }
                        }
                    }
                }
            }


            if (null != pourSlot) {
                var lcInfo = (LiquidContainerInfo)__instance.m_Info;

                if (pourSlot.gameObject.activeSelf) {
                    if (!Inventory3DManager.Get().m_CarriedItem) {
                        pourSlot.gameObject.SetActive(false);
                        return false;
                    }

                    if (((LiquidContainerInfo)Inventory3DManager.Get().m_CarriedItem.m_Info).m_Amount < 1f) {
                        pourSlot.gameObject.SetActive(false);
                        return false;
                    }

                    if (!pourSlot.CanInsertItem(Inventory3DManager.Get().m_CarriedItem)) {
                        pourSlot.gameObject.SetActive(false);
                        return false;
                    }

                    if (lcInfo.m_Amount >= lcInfo.m_Capacity) {
                        pourSlot.gameObject.SetActive(false);
                        return false;
                    }
                }
                else {
                    if (pourSlot.CanInsertItem(Inventory3DManager.Get().m_CarriedItem)) {
                        if (lcInfo.m_Amount < lcInfo.m_Capacity) {
                            pourSlot.gameObject.SetActive(true);
                        }
                    }
                }
            }


            return false;
        }
    }

    /// <summary>
    /// Method called when liquid is being drank from a liquid container to combine the stat effects for platyer.
    /// </summary>
    internal class Update_DrinkBidon_Patch {
        /// <summary>
        /// Prefix method called prior to the original drink liquid implementation.
        /// </summary>
        /// <param name="__instance">The original eating controller instance.</param>
        /// <param name="liquidType">The type of liquid being drank.</param>
        /// <param name="amount">The amound of the liquid drank (hydration)</param>
        /// <returns>False</returns>
        [HarmonyPrefix]
        public static bool Prefix(EatingController __instance, LiquidType liquidType, float amount) {
            LiquidData liquidData = LiquidManager.Get().GetLiquidData(liquidType);
            float dehydration = amount / 10f;
            float hydration = amount / 100f * liquidData.m_Water;
            float fats = liquidData.m_Fat * dehydration;
            float protein = liquidData.m_Proteins * dehydration;
            float carbs = liquidData.m_Carbohydrates * dehydration;
            float energy = liquidData.m_Energy * dehydration;

            PlayerConditionModule.Get().IncreaseHydration(hydration);
            PlayerConditionModule.Get().IncreaseNutritionFat(fats);
            PlayerConditionModule.Get().IncreaseNutritionProteins(protein);
            PlayerConditionModule.Get().IncreaseNutritionCarbo(carbs);
            PlayerConditionModule.Get().IncreaseEnergy(energy);
            PlayerSanityModule.Get().OnConsume((int)Math.Round(liquidData.m_SanityChange * dehydration));


            Debug.Log($"Bidon Prefix OnDrink: D:{dehydration}, H:{hydration}, F:{fats}, P:{protein}, C:{carbs}, E:{energy}");

            Update_DrinkBidon_Patch.OnDrink.Invoke(__instance, new object[] {
                liquidData,
                hydration,
                dehydration,
                fats,
                protein,
                carbs,
                energy
            });

            return false;
        }

        /// <summary>
        /// Reference to the original eating controller OnDrink method implementation.
        /// </summary>
        private static readonly MethodInfo OnDrink = typeof(EatingController).GetMethod("OnDrink", System.Reflection.BindingFlags.Instance | System.Reflection.BindingFlags.NonPublic);
    }
}
